package com.swagger.rest.services

import com.swagger.rest.models.Bug
import com.swagger.rest.models.BugInput
import com.swagger.rest.models.Enum
import com.swagger.rest.repositories.BugRepository
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.UserRepository
import jakarta.persistence.criteria.Predicate
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.data.jpa.domain.Specification
import org.springframework.http.HttpStatus
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Service
import java.util.*
import java.util.regex.Pattern

@Service
class BugService {

    @Autowired
    private val userRepository: UserRepository? = null

    @Autowired
    private val memberRepository: MemberRepository? = null

    @Autowired
    private val bugRepository: BugRepository? = null

    @Autowired
    private val platformRepository: PlatformRepository? = null

    private fun regex(input: String): Boolean {
//      "(http:\\/\\/|https:\\/\\/)?(www.)?([a-zA-Z0-9]+).[a-zA-Z0-9]*.[a-z]{3}.?([a-z]+)?"
//      "^(http:\\/\\/|https:\\/\\/)?(www.)?([a-zA-Z0-9]+).[a-zA-Z0-9]*.[a-z]{3}.?([a-z]+)?\$"
//      "(https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z0-9]{2,}(\.[a-zA-Z0-9]{2,})(\.[a-zA-Z0-9]{2,})?"
//      "(https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z0-9]{2,}(\.[a-zA-Z0-9]{2,})(\.[a-zA-Z0-9]{2,})?\/[a-zA-Z0-9]{2,}"
//      "(https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z0-9]{2,}\.[a-zA-Z0-9]{2,}\.[a-zA-Z0-9]{2,}(\.[a-zA-Z0-9]{2,})?"
//      "(https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z]{2,}(\.[a-zA-Z]{2,})(\.[a-zA-Z]{2,})?\/[a-zA-Z0-9]{2,}|((https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z]{2,}(\.[a-zA-Z]{2,})(\.[a-zA-Z]{2,})?)|(https:\/\/www\.|http:\/\/www\.|https:\/\/|http:\/\/)?[a-zA-Z0-9]{2,}\.[a-zA-Z0-9]{2,}\.[a-zA-Z0-9]{2,}(\.[a-zA-Z0-9]{2,})?"

        val regex = "(http://|https://)?(www.)?([a-zA-Z0-9]+).[a-zA-Z0-9]*.[a-z]{3}.?([a-z]+)?"
        val pattern = Pattern.compile(regex, Pattern.MULTILINE)
        val matcher = pattern.matcher(input)
        var ret = false
        while (matcher.find()) {
            ret = true
        }
        return ret
    }

    fun saveBug(bugInput: BugInput): Bug? {
        try {
            val userId = userRepository!!.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val save = Bug()
            save.created = Date()
            save.description = bugInput.description.trim()
            save.qc = userRepository.findById(bugInput.qc).get()
            save.dev = userRepository.findById(bugInput.dev).get()
            save.platform = platformRepository!!.findPlatform(userId.id.toString(), bugInput.platform)
            save.goodday_url = bugInput.goodday_url.trim()
            save.image_url = bugInput.image_url.trim()
            save.level = Enum.Level.valueOf(bugInput.level.uppercase()).ordinal
            save.status = Enum.Status.valueOf(bugInput.status.uppercase()).ordinal
            save.dev_status = Enum.Dev_Status.valueOf(bugInput.dev_status.uppercase()).ordinal
            return bugRepository!!.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            return null
        }
    }

    fun validate(bugInput: BugInput): HttpStatus? {
        try {
            val userId = userRepository!!.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
            val specBug = Specification<Bug> { root, _, builder ->
                val list: MutableList<Predicate> = mutableListOf()
                list.add(builder.equal(root.get<Bug>("description"), bugInput.description.trim()))
                builder.and(*list.toTypedArray())
            }
            val found = bugRepository!!.count(specBug)
            val countPlat = platformRepository!!.findPlatform2(userId.id.toString(), bugInput.platform)
            val countQc =
                memberRepository!!.validRoleByPlatform(bugInput.qc.toString(), bugInput.platform, 0.toString())
            val countDev =
                memberRepository.validRoleByPlatform(bugInput.dev.toString(), bugInput.platform, 1.toString())
            return if (countPlat > 0) {
                if (!regex(bugInput.goodday_url.trim()) || (bugInput.image_url.isNotEmpty() && !regex(bugInput.image_url.trim()))) {
                    HttpStatus.BAD_REQUEST
                } else if (!enumValues<Enum.Level>().any {
                        it.name == bugInput.level.uppercase()
                    } || !enumValues<Enum.Status>().any {
                        it.name == bugInput.status.uppercase()
                    } || !enumValues<Enum.Dev_Status>().any { it.name == bugInput.dev_status.uppercase() }) {
                    HttpStatus.BAD_REQUEST
                } else if (bugInput.description.isBlank()) {
                    HttpStatus.BAD_REQUEST
                } else if (bugInput.description.length > 500) {
                    HttpStatus.PAYLOAD_TOO_LARGE
                } else if (!(countQc == 0 || countDev == 0)) {
                    HttpStatus.FORBIDDEN
                } else if (found > 0) {
                    HttpStatus.CONFLICT
                } else {
                    HttpStatus.OK
                }
            } else {
                HttpStatus.NOT_FOUND
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return HttpStatus.INTERNAL_SERVER_ERROR
        }
    }

}