package com.swagger.rest.services

import com.swagger.rest.models.Platform
import com.swagger.rest.models.PlatformInput
import com.swagger.rest.repositories.MemberRepository
import com.swagger.rest.repositories.PlatformRepository
import com.swagger.rest.repositories.ProjectRepository
import com.swagger.rest.repositories.UserRepository
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.HttpStatus
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.stereotype.Service

@Service
class PlatformService {

    @Autowired
    private val platformRepository: PlatformRepository? = null

    @Autowired
    private val projectRepository: ProjectRepository? = null

    @Autowired
    private val userRepository: UserRepository? = null

    @Autowired
    private val memberRepository: MemberRepository? = null

    fun savePlatform(platformInput: PlatformInput): Platform? {
        try {
            val save = Platform()
            save.name = platformInput.name!!.trim()
            save.project = projectRepository!!.findByName(platformInput.project_name)
            return platformRepository!!.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            return null
        }
    }

    fun updatePlatform(id:Long, platformInput: PlatformInput): Platform? {
        try {
            val save = platformRepository!!.findById(id).get()
            save.name = platformInput.name!!.trim()
            save.project = projectRepository!!.findByName(platformInput.project_name)
            return platformRepository.save(save)
        } catch (e: Exception) {
            e.printStackTrace()
            return null
        }
    }

    fun validate(platformInput: PlatformInput): HttpStatus? {
        try {
            val proj = projectRepository!!.countByName(platformInput.project_name) > 0
            return if (proj && platformInput.project_name!!.isNotBlank()) {
                val foundProject = projectRepository.findByName(platformInput.project_name)
                val found = platformRepository!!.findByName(
                    platformInput.name, foundProject.id.toString()
                ).size
                val userId = userRepository!!.getUserByUsername(SecurityContextHolder.getContext().authentication.name)
                val validOwner = projectRepository.validOwner(foundProject.id.toString(), userId.id.toString())
                val validAdmin = memberRepository!!.validRole(foundProject.id.toString(), userId.id.toString(), "2")
                if (platformInput.name!!.isBlank()) {
                    HttpStatus.BAD_REQUEST
                } else if (!(validOwner > 0 || validAdmin > 0)) {
                    HttpStatus.FORBIDDEN
                } else if (platformInput.name!!.length > 100) {
                    HttpStatus.PAYLOAD_TOO_LARGE
                } else if (found > 0) {
                    HttpStatus.CONFLICT
                } else {
                    HttpStatus.OK
                }
            } else {
                HttpStatus.NOT_FOUND
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return HttpStatus.INTERNAL_SERVER_ERROR
        }
    }

}